@@ -76,7 +76,6 @@ static void
 eog_image_dispose (GObject *object)
 {
 	EogImagePrivate *priv;
-	GList *it;
 
 	priv = EOG_IMAGE (object)->priv;
 
@@ -106,6 +105,11 @@ eog_image_dispose (GObject *object)
 		g_free (priv->error_message);
 		priv->error_message = NULL;
 	}
+
+	if (priv->mutex) {
+		g_mutex_free (priv->mutex);
+		priv->mutex = NULL;
+	}
 	
 	if (priv->status_mutex) {
 		g_mutex_free (priv->status_mutex);
@@ -118,11 +122,8 @@ eog_image_dispose (GObject *object)
 	}
 
 	if (priv->undo_stack) {
-		for (it = priv->undo_stack; it != NULL; it = it->next){
-			g_object_unref (G_OBJECT (it->data));
-		}
-
-		g_list_free (priv->undo_stack);
+		g_slist_foreach (priv->undo_stack, (GFunc) g_object_unref, NULL);
+		g_slist_free (priv->undo_stack);
 		priv->undo_stack = NULL;
 	}
 }
@@ -257,6 +258,7 @@ eog_image_init (EogImage *img)
 	priv->thumbnail = NULL;
 	priv->width = priv->height = -1;
 	priv->modified = FALSE;
+	priv->mutex = g_mutex_new ();
 	priv->status_mutex = g_mutex_new ();
 	priv->load_finished = NULL;
 	priv->error_message = NULL;
@@ -271,6 +273,24 @@ eog_image_init (EogImage *img)
 	img->priv = priv;
 }
 
+void eog_image_lock (EogImage *image)
+{
+        g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (image->priv != NULL);
+	g_return_if_fail (image->priv->mutex != NULL);
+
+	g_mutex_lock (image->priv->mutex);
+}
+
+void eog_image_unlock (EogImage *image)
+{
+        g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (image->priv != NULL);
+	g_return_if_fail (image->priv->mutex != NULL);
+
+	g_mutex_unlock (image->priv->mutex);
+}
+
 EogImage* 
 eog_image_new_uri (GnomeVFSURI *uri)
 {
@@ -463,107 +483,6 @@ eog_image_determine_file_bytes (EogImage *img, GError **error)
 	return bytes;
 }
 
-/* this function runs in it's own thread */
-static gboolean
-eog_image_load_exif_data_only (EogImage *img, EogJob *job, GError **error)
-{
-	EogImagePrivate *priv;
-	GnomeVFSHandle *handle;
-	guchar *buffer;
-	GnomeVFSFileSize bytes_read;
-	GnomeVFSFileSize bytes_read_total;
-	GnomeVFSResult result;
-	gboolean failed;
-	gboolean first_run = TRUE;
-	EogMetadataReader *md_reader = NULL;
-
-	g_assert (error == NULL || *error == NULL);
-
-	priv = img->priv;
-
-#ifdef DEBUG
-	g_print ("real image exif load %s\n", gnome_vfs_uri_to_string (priv->uri, GNOME_VFS_URI_HIDE_NONE));
-#endif
-
-	g_assert (priv->image == NULL);
-
-	priv->bytes = eog_image_determine_file_bytes (img, error);
-	if (priv->bytes == 0 && (error == NULL || *error != NULL)) {
-		return FALSE;
-	}
-
-	result = gnome_vfs_open_uri (&handle, priv->uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
-		g_set_error (error, EOG_IMAGE_ERROR, EOG_IMAGE_ERROR_VFS,
-			     gnome_vfs_result_to_string (result));
-		return FALSE;
-	}
-	
-	buffer = g_new0 (guchar, READ_BUFFER_SIZE);
-	failed = FALSE;
-	bytes_read_total = 0;
-
-	while (!priv->cancel_loading && (first_run || !eog_metadata_reader_finished (md_reader))) {
-
-		result = gnome_vfs_read (handle, buffer, READ_BUFFER_SIZE, &bytes_read);
-		if (result == GNOME_VFS_ERROR_EOF || bytes_read == 0) {
-			break;
-		}
-		else if (result != GNOME_VFS_OK) {
-			failed = TRUE;
-			g_set_error (error, EOG_IMAGE_ERROR, EOG_IMAGE_ERROR_VFS,
-				     gnome_vfs_result_to_string (result));
-			break;
-		}
-		
-		bytes_read_total += bytes_read;
-
-		if (job != NULL) {
-			float progress = (float) bytes_read_total / (float) priv->bytes;
-			eog_job_set_progress (job, progress);
-		}
-		
-		/* check if we support reading metadata for that image format (only JPG atm) */
-		if (first_run) {
-			md_reader = check_for_metadata_img_format (img, buffer, bytes_read);
-			g_set_error (error, EOG_IMAGE_ERROR, EOG_IMAGE_ERROR_GENERIC,
-				     _("EXIF not supported for this file format."));
-			first_run = FALSE;
-		}
-		
-		if (md_reader != NULL) {
-			eog_metadata_reader_consume (md_reader, buffer, bytes_read);
-		}
-	}
-
-	g_free (buffer);
-	gnome_vfs_close (handle);
-	
-	failed = (failed || bytes_read_total == 0 || md_reader == NULL || priv->cancel_loading);
-
-	if (priv->cancel_loading) {
-		priv->cancel_loading = FALSE;
-		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
-	}
-	else if (!failed && (md_reader != NULL)) {
-		/* update meta data */
-#ifdef HAVE_EXIF  
-		priv->exif = eog_metadata_reader_get_exif_data (md_reader);
-		priv->exif_chunk = NULL;
-		priv->exif_chunk_len = 0;
-#else
-		eog_metadata_reader_get_exif_chunk (md_reader, &priv->exif_chunk, &priv->exif_chunk_len);
-#endif
-	}
-	
-	/* clean up */
-	if (md_reader != NULL) {
-		g_object_unref (md_reader);
-	}	
-
-	return !failed;
-}
-
 #ifdef HAVE_LCMS
 void
 eog_image_apply_display_profile (EogImage *img, cmsHPROFILE screen)
@@ -613,9 +532,13 @@ extract_profile (EogImage *img, EogMetadataReader *md_reader)
 			cmsErrorAction (LCMS_ERROR_SHOW);
 			priv->profile = cmsOpenProfileFromMem(data, eog_metadata_reader_get_icc_chunk_size (md_reader));
 			if (priv->profile) {
+#ifdef DEBUG	
 				g_printerr("JPEG has ICC profile\n");
+#endif
 			} else {
+#ifdef DEBUG	
 				g_printerr("JPEG has invalid ICC profile\n");
+#endif
 			}
 			return;
 		}
@@ -631,11 +554,15 @@ extract_profile (EogImage *img, EogMetadataReader *md_reader)
 
 	if (exif_get_short (entry->data, o) == 1) {
 		priv->profile = cmsCreate_sRGBProfile ();
+#ifdef DEBUG	
 		g_printerr ("JPEG is sRGB\n");
+#endif
 	} else if (exif_get_short (entry->data, o) == 2) {
 		/* TODO: create Adobe RGB profile */
 		//priv->profile = cmsCreate_Adobe1998Profile ();
+#ifdef DEBUG	
 		g_printerr ("JPEG is Adobe RGB (NOT correcting for now!)\n");
+#endif
 	} else if (exif_get_short (entry->data, o) == 0xFFFF) {
 		double gammaValue;
 		cmsCIExyY whitepoint;
@@ -693,7 +620,9 @@ extract_profile (EogImage *img, EogMetadataReader *md_reader)
 		gamma[0] = gamma[1] = gamma[2] = cmsBuildGamma(256, gammaValue);
 		    
 		priv->profile = cmsCreateRGBProfile(&whitepoint, &primaries, gamma);
+#ifdef DEBUG	
 		g_printerr ("JPEG is calibrated\n");
+#endif
 		cmsFreeGamma(gamma[0]);
 	}
 #endif
@@ -705,7 +634,7 @@ static gboolean
 eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error)
 {
 	EogImagePrivate *priv;
-	GdkPixbufLoader *loader;
+	GdkPixbufLoader *loader = NULL;
 	GnomeVFSHandle *handle;
 	guchar *buffer;
 	GnomeVFSFileSize bytes_read;
@@ -715,6 +644,7 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	gboolean first_run = TRUE;
 	EogMetadataReader *md_reader = NULL;
 	GdkPixbufFormat *format;
+	gboolean read_image_data = (data2read & EOG_IMAGE_DATA_IMAGE);
 
 	g_assert (error == NULL || *error == NULL);
 
@@ -726,7 +656,7 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 
 	g_assert (priv->image == NULL);
 
-	if (priv->file_type != NULL) {
+	if (read_image_data && priv->file_type != NULL) {
 		g_free (priv->file_type);
 		priv->file_type = NULL;
 	}
@@ -744,11 +674,13 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	}
 	
 	buffer = g_new0 (guchar, READ_BUFFER_SIZE);
-	loader = gdk_pixbuf_loader_new ();
 	failed = FALSE;
 	bytes_read_total = 0;
 
-	g_signal_connect_object (G_OBJECT (loader), "size-prepared", (GCallback) load_size_prepared, img, 0);
+	if (read_image_data) {
+		loader = gdk_pixbuf_loader_new ();
+		g_signal_connect_object (G_OBJECT (loader), "size-prepared", (GCallback) load_size_prepared, img, 0);
+        }
 
 	while (!priv->cancel_loading) {
 
@@ -763,7 +695,7 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 			break;
 		}
 		
-		if (!gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
+		if (read_image_data && !gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
 			failed = TRUE;
 			break;
 		}
@@ -778,21 +710,31 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 		/* check if we support reading metadata for that image format (only JPG atm) */
 		if (first_run) {
 			md_reader = check_for_metadata_img_format (img, buffer, bytes_read);
+			if (data2read == EOG_IMAGE_DATA_EXIF && md_reader == NULL) {
+				g_set_error (error, EOG_IMAGE_ERROR, 
+                                             EOG_IMAGE_ERROR_GENERIC,
+                                             _("EXIF not supported for this file format."));
+				break;
+                        }
 			first_run = FALSE;
 		}
 		
 		if (md_reader != NULL) {
 			eog_metadata_reader_consume (md_reader, buffer, bytes_read);
+			if (data2read == EOG_IMAGE_DATA_EXIF && eog_metadata_reader_finished (md_reader))
+				break;
 		}
 	}
 
-	/* if we already failed ignore errors on close */
-	if (failed) {
-		gdk_pixbuf_loader_close (loader, NULL);
-	}
-	else if (!gdk_pixbuf_loader_close (loader, error)) {
-		failed = TRUE;
-	}
+	if (read_image_data) {
+		/* if we already failed ignore errors on close */
+		if (failed) {
+			gdk_pixbuf_loader_close (loader, NULL);
+		}
+		else if (!gdk_pixbuf_loader_close (loader, error)) {
+			failed = TRUE;
+		}
+        }
 
 	g_free (buffer);
 	gnome_vfs_close (handle);
@@ -811,17 +753,24 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	}
 	else if (!failed) {
 
-		/* update image data */
-		if (priv->image != NULL) {
-			g_object_unref (priv->image);
+		if (read_image_data) {
+			/* update image data */
+			if (priv->image != NULL) {
+				g_object_unref (priv->image);
+			}
+
+			priv->image = gdk_pixbuf_loader_get_pixbuf (loader);
+			g_assert (priv->image != NULL);
+			g_object_ref (priv->image);		
+			priv->width = gdk_pixbuf_get_width (priv->image);
+			priv->height = gdk_pixbuf_get_height (priv->image);
+
+			/* update file format */
+			format = gdk_pixbuf_loader_get_format (loader);
+			if (format != NULL) {
+				priv->file_type = gdk_pixbuf_format_get_name (format);
+			}
 		}
-
-		priv->image = gdk_pixbuf_loader_get_pixbuf (loader);
-		g_assert (priv->image != NULL);
-		g_object_ref (priv->image);		
-		priv->width = gdk_pixbuf_get_width (priv->image);
-		priv->height = gdk_pixbuf_get_height (priv->image);
-
 		/* update meta data */
 		if (md_reader != NULL) {
 #ifdef HAVE_EXIF  
@@ -834,11 +783,6 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 #endif
 		}
 
-		/* update file format */
-		format = gdk_pixbuf_loader_get_format (loader);
-		if (format != NULL) {
-			priv->file_type = gdk_pixbuf_format_get_name (format);
-		}
 #ifdef HAVE_LCMS
 		if (md_reader != NULL) {
 			extract_profile (img, md_reader);
@@ -846,7 +790,8 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 #endif
 	}
 	/* clean up */
-	g_object_unref (loader);
+	if (loader != NULL)
+		g_object_unref (loader);
 	if (md_reader != NULL) {
 		g_object_unref (md_reader);
 		md_reader = NULL;
@@ -932,12 +877,7 @@ eog_image_load (EogImage *img, guint data2read, EogJob *job, GError **error)
 	}
 
 	/* Read the requested data from the image */
-	if (data2read == EOG_IMAGE_DATA_EXIF) {
-		success = eog_image_load_exif_data_only (img, job, error);
-	}
-	else {
-		success = eog_image_real_load (img, data2read, job, error);
-	}
+	success = eog_image_real_load (img, data2read, job, error);
 
 #ifdef DEBUG
 	g_print ("load success: %i\n", success);
@@ -1110,7 +1050,7 @@ image_transform (EogImage *img, EogTransform *trans, gboolean is_undo, EogJob *j
 	
 	if (!is_undo) {
 		g_object_ref (trans);
-		priv->undo_stack = g_list_prepend (priv->undo_stack, trans);
+		priv->undo_stack = g_slist_prepend (priv->undo_stack, trans);
 	}
 }
 
@@ -1139,7 +1079,7 @@ eog_image_undo (EogImage *img)
 
 		image_transform (img, inverse, TRUE, NULL);
 
-		priv->undo_stack = g_list_delete_link (priv->undo_stack, priv->undo_stack);
+		priv->undo_stack = g_slist_delete_link (priv->undo_stack, priv->undo_stack);
 		g_object_unref (trans);
 		g_object_unref (inverse);
 
@@ -1263,17 +1203,13 @@ static void
 eog_image_reset_modifications (EogImage *image)
 {
 	EogImagePrivate *priv;
-	GList *it = NULL;
 
 	g_return_if_fail (EOG_IS_IMAGE (image));
 
 	priv = image->priv;
 
-	/* free the undo stack */
-	for (it = priv->undo_stack; it != NULL; it = it->next) {
-		g_object_unref (G_OBJECT (it->data));
-	}
-	g_list_free (priv->undo_stack);
+	g_slist_foreach (priv->undo_stack, (GFunc) g_object_unref, NULL);
+	g_slist_free (priv->undo_stack);
 	priv->undo_stack = NULL;
 
 	/* free accumulated transform object */
@@ -1692,6 +1628,7 @@ eog_image_cancel_load (EogImage *img)
 	g_mutex_lock (priv->status_mutex);
 	if (priv->status == EOG_IMAGE_STATUS_LOADING) {
 		priv->cancel_loading = TRUE;
+		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 	}
 	g_mutex_unlock (priv->status_mutex);
 }
